# -*- coding: utf-8 -*-
#
# Copyright © Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see spyder/__init__.py for details)

"""
Find in files widget API.
"""

# Local imports
from spyder.plugins.findinfiles.plugin import FindInFilesActions
from spyder.plugins.findinfiles.widgets import FindInFilesWidgetActions
