[{"locale":"es"},{"key":"03D0","mappings":{"default":{"default":"símbolo beta"}},"category":"Ll"},{"key":"03D1","mappings":{"default":{"default":"variante de theta"}},"category":"Ll"},{"key":"03D5","mappings":{"default":{"default":"phi recta"}},"category":"Ll"},{"key":"03D6","mappings":{"default":{"default":"pi","defaultMP":"variante de pi"}},"category":"Ll"},{"key":"03D7","mappings":{"default":{"default":"símbolo kai griego"}},"category":"Ll"},{"key":"03F0","mappings":{"default":{"default":"kappa","defaultMP":"variante de kappa"}},"category":"Ll"},{"key":"03F1","mappings":{"default":{"default":"rho","defaultMP":"variante de rho"}},"category":"Ll"},{"key":"03F5","mappings":{"default":{"default":"épsilon","defaultMP":"variante de épsilon"}},"category":"Ll"},{"key":"03F6","mappings":{"default":{"default":"épsilon invertido"}},"category":"Sm"},{"key":"03F4","mappings":{"default":{"default":"mayúscula variante de Theta"}},"category":"Lu"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula variante de Theta"}},"key":"1D6B9"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula variante de Theta"}},"key":"1D6F3"},{"category":"Lu","mappings":{"default":{"default":"sans-serif negrita mayúscula variante de Theta"}},"key":"1D767"}]
